#include <Keypad.h>
#include <Servo.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2); 

Servo myservo;  // create servo object to control a servo


const byte ROWS = 4;
const byte COLS = 4;
int buzzer = A3;
int blue = A2;
int green = A1;
int red = A0;

char hexaKeys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'0', 'F', 'E', 'D'}
};
byte rowPins[ROWS] = {5, 4, 3, 2};
byte colPins[COLS] = {9, 8, 7, 6};
long first = 0;

Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);

void setup() {

  lcd.init();           
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("Select servo pos");
  lcd.setCursor(0, 1);
  lcd.print("0~180,D=run,in=0");

  myservo.attach(10);  // attaches the servo on pin 10 to the servo object
  
  pinMode(buzzer, OUTPUT);
  pinMode(red, OUTPUT);
  pinMode(green, OUTPUT);
  pinMode(blue, OUTPUT);
  
  ok();

}


void loop() {
  char Key = customKeypad.getKey();
  switch (Key) {

    case '0' ... '9':
      ok();
      lcd.clear();
      lcd.setCursor(0, 0);
      first = first * 10 + (Key - '0'); //keeps collecting input values to create a multi-digit number
      lcd.print(first);
      lcd.print(" deg");
      lcd.setCursor(9, 1);
      lcd.print("D = run");
      break;

    case 'D':
      if (first > 180) {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("ERR_exceeded_lim");
        error();

        delay(1500);
        first = 0;
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Select servo pos");
        lcd.setCursor(0, 1);
        lcd.print("0~180,D=run,in=0");
      } else {

        ok();
        myservo.write(first);
        Serial.println(first);
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Running ");
        lcd.print(first);
        lcd.print(" deg");
        lcd.setCursor(0, 1);
        lcd.print("New input req'd");
        first = 0;


      }
      break;
    case 'A':
      info();
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Help-servo ~ D10");
      lcd.setCursor(0, 1);
      lcd.print("'C' key to clear");

      break;
    case 'C':
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Select servo pos");
      lcd.setCursor(0, 1);
      lcd.print("0~180,D=run,in=0");
      ok();
  }

}

/* 
Just a dew methods to simplify code- they are used for controlling the LED and buzzer based on different events.
*/

void ok() {
  digitalWrite(blue, LOW);
  digitalWrite(red, LOW);
  digitalWrite(green, HIGH);

  digitalWrite(buzzer, HIGH);
  delay(20);
  digitalWrite(buzzer, LOW);

}
void info() {
  digitalWrite(blue, HIGH);
  digitalWrite(red, LOW);
  digitalWrite(green, LOW);

}
void error() {
  digitalWrite(blue, LOW);
  digitalWrite(red, HIGH);
  digitalWrite(green, LOW);
  digitalWrite(buzzer, HIGH);
  delay(1000);
  digitalWrite(buzzer, LOW);
  digitalWrite(red, LOW);
  digitalWrite(green, HIGH);
}
